@@ -9,7 +9,7 @@ __all__ = ['handler404', 'handler500']
 admin.autodiscover()
 
 urlpatterns = [
-    url(r'^admin/', include(admin.site.urls)),
+    url(r'^admin/', admin.site.urls),
     url(r'^logout/$', logout, {'next_page': '/'}, name='logout'),
     url(r'^article/', include('articles.urls', namespace='articles')),
     url(r'^', include('posts.urls')),
