@@ -1,9 +1,9 @@
 from __future__ import unicode_literals
 from django.contrib.auth.models import Permission
-from django.contrib.contenttypes.models import ContentType
 from django.db.models.query import QuerySet
 from django.utils.encoding import force_text
 from guardian.compat import get_user_model
+from guardian.ctypes import get_content_type
 from guardian.utils import get_group_obj_perms_model, get_identity, get_user_obj_perms_model
 from itertools import chain
 
@@ -17,13 +17,13 @@ def _get_pks_model_and_ctype(objects):
     if isinstance(objects, QuerySet):
         model = objects.model
         pks = [force_text(pk) for pk in objects.values_list('pk', flat=True)]
-        ctype = ContentType.objects.get_for_model(model)
+        ctype = get_content_type(model)
     else:
         pks = []
         for idx, obj in enumerate(objects):
             if not idx:
                 model = type(obj)
-                ctype = ContentType.objects.get_for_model(model)
+                ctype = get_content_type(model)
             pks.append(force_text(obj.pk))
 
     return pks, model, ctype
@@ -76,7 +76,7 @@ class ObjectPermissionChecker(object):
 
     def get_group_filters(self, obj):
         User = get_user_model()
-        ctype = ContentType.objects.get_for_model(obj)
+        ctype = get_content_type(obj)
 
         group_model = get_group_obj_perms_model(obj)
         group_rel_name = group_model.permission.field.related_query_name()
@@ -99,7 +99,7 @@ class ObjectPermissionChecker(object):
         return group_filters
 
     def get_user_filters(self, obj):
-        ctype = ContentType.objects.get_for_model(obj)
+        ctype = get_content_type(obj)
         model = get_user_obj_perms_model(obj)
         related_name = model.permission.field.related_query_name()
 
@@ -115,7 +115,7 @@ class ObjectPermissionChecker(object):
         return user_filters
 
     def get_user_perms(self, obj):
-        ctype = ContentType.objects.get_for_model(obj)
+        ctype = get_content_type(obj)
 
         perms_qs = Permission.objects.filter(content_type=ctype)
         user_filters = self.get_user_filters(obj)
@@ -125,7 +125,7 @@ class ObjectPermissionChecker(object):
         return user_perms
 
     def get_group_perms(self, obj):
-        ctype = ContentType.objects.get_for_model(obj)
+        ctype = get_content_type(obj)
 
         perms_qs = Permission.objects.filter(content_type=ctype)
         group_filters = self.get_group_filters(obj)
@@ -143,7 +143,7 @@ class ObjectPermissionChecker(object):
         """
         if self.user and not self.user.is_active:
             return []
-        ctype = ContentType.objects.get_for_model(obj)
+        ctype = get_content_type(obj)
         key = self.get_local_cache_key(obj)
         if key not in self._obj_perms_cache:
             if self.user and self.user.is_superuser:
@@ -169,7 +169,7 @@ class ObjectPermissionChecker(object):
         """
         Returns cache key for ``_obj_perms_cache`` dict.
         """
-        ctype = ContentType.objects.get_for_model(obj)
+        ctype = get_content_type(obj)
         return (ctype.id, force_text(obj.pk))
 
     def prefetch_perms(self, objects):
