@@ -2,18 +2,23 @@
 Convenient shortcuts to manage or check object permissions.
 """
 from __future__ import unicode_literals
+
+import warnings
+from collections import defaultdict
+from itertools import groupby
+
 from django.apps import apps
 from django.contrib.auth.models import Group, Permission
 from django.contrib.contenttypes.models import ContentType
 from django.db.models import Count, Q, QuerySet
 from django.shortcuts import _get_queryset
-from guardian.compat import basestring, get_user_model
-from guardian.core import ObjectPermissionChecker
-from guardian.exceptions import MixedContentTypeError, WrongAppError
-from guardian.utils import get_anonymous_user, get_group_obj_perms_model, get_identity, get_user_obj_perms_model
-from itertools import groupby
 
-import warnings
+from guardian.compat import basestring, get_user_model, is_anonymous
+from guardian.core import ObjectPermissionChecker
+from guardian.ctypes import get_content_type
+from guardian.exceptions import MixedContentTypeError, WrongAppError
+from guardian.models import GroupObjectPermission
+from guardian.utils import get_anonymous_user, get_group_obj_perms_model, get_identity, get_user_obj_perms_model
 
 
 def assign_perm(perm, user_or_group, obj=None):
@@ -106,7 +111,9 @@ def assign_perm(perm, user_or_group, obj=None):
 
 def assign(perm, user_or_group, obj=None):
     """ Depreciated function name left in for compatibility"""
-    warnings.warn("Shortcut function 'assign' is being renamed to 'assign_perm'. Update your code accordingly as old name will be depreciated in 2.0 version.", DeprecationWarning)
+    warnings.warn(
+        "Shortcut function 'assign' is being renamed to 'assign_perm'. Update your code accordingly as old name will be depreciated in 2.0 version.",
+        DeprecationWarning)
     return assign_perm(perm, user_or_group, obj)
 
 
@@ -200,7 +207,7 @@ def get_perms_for_model(cls):
         model = apps.get_model(app_label, model_name)
     else:
         model = cls
-    ctype = ContentType.objects.get_for_model(model)
+    ctype = get_content_type(model)
     return Permission.objects.filter(content_type=ctype)
 
 
@@ -240,7 +247,7 @@ def get_users_with_perms(obj, attach_perms=False, with_superusers=False,
         {<User: joe>: [u'change_flatpage']}
 
     """
-    ctype = ContentType.objects.get_for_model(obj)
+    ctype = get_content_type(obj)
     if not attach_perms:
         # It's much easier without attached perms so we do it first if that is
         # the case
@@ -312,11 +319,11 @@ def get_groups_with_perms(obj, attach_perms=False):
         {<Group: admins>: [u'change_flatpage']}
 
     """
-    ctype = ContentType.objects.get_for_model(obj)
+    ctype = get_content_type(obj)
+    group_model = get_group_obj_perms_model(obj)
+
     if not attach_perms:
-        # It's much easier without attached perms so we do it first if that is
-        # the case
-        group_model = get_group_obj_perms_model(obj)
+        # It's much easier without attached perms so we do it first if that is the case
         group_rel_name = group_model.group.field.related_query_name()
         if group_model.objects.is_generic():
             group_filters = {
@@ -325,15 +332,19 @@ def get_groups_with_perms(obj, attach_perms=False):
             }
         else:
             group_filters = {'%s__content_object' % group_rel_name: obj}
-        groups = Group.objects.filter(**group_filters).distinct()
-        return groups
+        return Group.objects.filter(**group_filters).distinct()
     else:
-        # TODO: Do not hit db for each group!
-        groups = {}
-        for group in get_groups_with_perms(obj):
-            if group not in groups:
-                groups[group] = sorted(get_group_perms(group, obj))
-        return groups
+        group_perms_mapping = defaultdict(list)
+        groups_with_perms = get_groups_with_perms(obj)
+        qs = group_model.objects.filter(group__in=groups_with_perms).prefetch_related('group', 'permission')
+        if group_model is GroupObjectPermission:
+            qs = qs.filter(object_pk=obj.pk)
+        else:
+            qs = qs.filter(content_object_id=obj.pk)
+
+        for group_perm in qs:
+            group_perms_mapping[group_perm.group].append(group_perm.permission.codename)
+        return dict(group_perms_mapping)
 
 
 def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=False,
@@ -342,9 +353,6 @@ def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=Fals
     Returns queryset of objects for which a given ``user`` has *all*
     permissions present at ``perms``.
 
-    If ``perms`` is an empty list, then it returns objects for which
-    a given ``user`` has *any* object permission.
-
     :param user: ``User`` or ``AnonymousUser`` instance for which objects would
       be returned.
     :param perms: single permission string, or sequence of permission strings
@@ -470,7 +478,7 @@ def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=Fals
     # Compute queryset and ctype if still missing
     if ctype is None and klass is not None:
         queryset = _get_queryset(klass)
-        ctype = ContentType.objects.get_for_model(queryset.model)
+        ctype = get_content_type(queryset.model)
     elif ctype is not None and klass is None:
         queryset = _get_queryset(ctype.model_class())
     elif klass is None:
@@ -492,7 +500,7 @@ def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=Fals
     # Check if the user is anonymous. The
     # django.contrib.auth.models.AnonymousUser object doesn't work for queries
     # and it's nice to be able to pass in request.user blindly.
-    if user.is_anonymous():
+    if is_anonymous(user):
         user = get_anonymous_user()
 
     global_perms = set()
@@ -552,7 +560,7 @@ def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=Fals
             group_fields = generic_fields
         else:
             group_fields = direct_fields
-        if not any_perm and len(codenames) and not has_global_perms:
+        if not any_perm and len(codenames) > 1 and not has_global_perms:
             user_obj_perms = user_obj_perms_queryset.values_list(*user_fields)
             groups_obj_perms = groups_obj_perms_queryset.values_list(*group_fields)
             data = list(user_obj_perms) + list(groups_obj_perms)
@@ -575,12 +583,12 @@ def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=Fals
 
     values = user_obj_perms_queryset.values_list(user_fields[0], flat=True)
     if user_model.objects.is_generic():
-        values = list(values)
+        values = set(values)
     q = Q(pk__in=values)
     if use_groups:
         values = groups_obj_perms_queryset.values_list(group_fields[0], flat=True)
         if group_model.objects.is_generic():
-            values = list(values)
+            values = set(values)
         q |= Q(pk__in=values)
 
     return queryset.filter(q)
@@ -673,7 +681,7 @@ def get_objects_for_group(group, perms, klass=None, any_perm=False, accept_globa
     # Compute queryset and ctype if still missing
     if ctype is None and klass is not None:
         queryset = _get_queryset(klass)
-        ctype = ContentType.objects.get_for_model(queryset.model)
+        ctype = get_content_type(queryset.model)
     elif ctype is not None and klass is None:
         queryset = _get_queryset(ctype.model_class())
     elif klass is None:
