@@ -4,6 +4,7 @@ from guardian.checks import check_settings
 
 
 class SystemCheckTestCase(TestCase):
+
     def test_checks(self):
         """ Test custom system checks
         :return: None
@@ -11,7 +12,7 @@ class SystemCheckTestCase(TestCase):
         self.assertFalse(check_settings(None))
 
         with self.settings(
-                AUTHENTICATION_BACKENDS=('django.contrib.auth.backends.ModelBackend',),
-                ANONYMOUS_USER_ID=None,
-                ):
-            self.assertEqual(len(check_settings(None)), 2)
+                AUTHENTICATION_BACKENDS=(
+                    'django.contrib.auth.backends.ModelBackend',),
+        ):
+            self.assertEqual(len(check_settings(None)), 1)
