@@ -42,9 +42,17 @@ class ObjectPermissionTestCase(TestCase):
         self.user.groups.add(self.group)
         self.ctype = ContentType.objects.create(
             model='bar', app_label='fake-for-guardian-tests')
+        self.ctype_qset = ContentType.objects.filter(model='bar',
+                                                     app_label='fake-for-guardian-tests')
         self.anonymous_user = User.objects.get(
             username=guardian_settings.ANONYMOUS_USER_NAME)
 
+    def get_permission(self, codename, app_label=None):
+        qs = Permission.objects
+        if app_label:
+            qs = qs.filter(content_type__app_label=app_label)
+        return Permission.objects.get(codename=codename)
+
 
 class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
 
@@ -190,18 +198,23 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             from django.db import connection
 
             ContentType.objects.clear_cache()
-            new_group = Group.objects.create(name='new-group')
+            group1 = Group.objects.create(name='group1')
+            group2 = Group.objects.create(name='group2')
             user = User.objects.create(username='active_user', is_active=True)
             assign_perm("change_group", user, self.group)
-            assign_perm("change_group", user, new_group)
+            assign_perm("change_group", user, group1)
             checker = ObjectPermissionChecker(user)
 
             # Prefetch permissions
-            self.assertTrue(checker.prefetch_perms([self.group, new_group]))
+            prefetched_objects = [self.group, group1, group2]
+            self.assertTrue(checker.prefetch_perms(prefetched_objects))
             query_count = len(connection.queries)
 
             # Checking cache is filled
-            self.assertEqual(len(checker._obj_perms_cache), 2)
+            self.assertEqual(
+                len(checker._obj_perms_cache),
+                len(prefetched_objects)
+            )
 
             # Checking shouldn't spawn any queries
             checker.has_perm("change_group", self.group)
@@ -213,9 +226,14 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             self.assertEqual(len(connection.queries), query_count)
 
             # Checking for same model but other instance shouldn't spawn any queries
-            checker.has_perm("change_group", new_group)
+            checker.has_perm("change_group", group1)
             self.assertEqual(len(connection.queries), query_count)
 
+            # Checking for same model but other instance shouldn't spawn any queries
+            # Even though User doesn't have perms on Group2, we still should
+            #  not hit DB
+            self.assertFalse(checker.has_perm("change_group", group2))
+            self.assertEqual(len(connection.queries), query_count)
         finally:
             settings.DEBUG = False
 
@@ -225,18 +243,22 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             from django.db import connection
 
             ContentType.objects.clear_cache()
-            new_group = Group.objects.create(name='new-group')
+            group1 = Group.objects.create(name='group1')
             user = User.objects.create(username='active_superuser',
                                        is_superuser=True, is_active=True)
             assign_perm("change_group", user, self.group)
             checker = ObjectPermissionChecker(user)
 
             # Prefetch permissions
-            self.assertTrue(checker.prefetch_perms([self.group, new_group]))
+            prefetched_objects = [self.group, group1]
+            self.assertTrue(checker.prefetch_perms(prefetched_objects))
             query_count = len(connection.queries)
 
             # Checking cache is filled
-            self.assertEqual(len(checker._obj_perms_cache), 2)
+            self.assertEqual(
+                len(checker._obj_perms_cache),
+                len(prefetched_objects)
+            )
 
             # Checking shouldn't spawn any queries
             checker.has_perm("change_group", self.group)
@@ -248,9 +270,8 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             self.assertEqual(len(connection.queries), query_count)
 
             # Checking for same model but other instance shouldn't spawn any queries
-            checker.has_perm("change_group", new_group)
+            checker.has_perm("change_group", group1)
             self.assertEqual(len(connection.queries), query_count)
-
         finally:
             settings.DEBUG = False
 
@@ -260,17 +281,23 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             from django.db import connection
 
             ContentType.objects.clear_cache()
-            new_group = Group.objects.create(name='new-group')
-            assign_perm("change_group", new_group, self.group)
-            assign_perm("change_group", new_group, new_group)
-            checker = ObjectPermissionChecker(new_group)
+            group1 = Group.objects.create(name='group1')
+            group2 = Group.objects.create(name='group2')
+            assign_perm("change_group", group1, self.group)
+            assign_perm("change_group", group1, group1)
+            checker = ObjectPermissionChecker(group1)
 
             # Prefetch permissions
-            self.assertTrue(checker.prefetch_perms([self.group, new_group]))
+            prefetched_objects = [self.group, group1, group2]
+            self.assertTrue(checker.prefetch_perms(prefetched_objects))
+
             query_count = len(connection.queries)
 
             # Checking cache is filled
-            self.assertEqual(len(checker._obj_perms_cache), 2)
+            self.assertEqual(
+                len(checker._obj_perms_cache),
+                len(prefetched_objects)
+            )
 
             # Checking shouldn't spawn any queries
             checker.has_perm("change_group", self.group)
@@ -282,9 +309,14 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             self.assertEqual(len(connection.queries), query_count)
 
             # Checking for same model but other instance shouldn't spawn any queries
-            checker.has_perm("change_group", new_group)
+            checker.has_perm("change_group", group1)
             self.assertEqual(len(connection.queries), query_count)
 
+            # Checking for same model but other instance shouldn't spawn any queries
+            # Even though User doesn't have perms on Group2, we still should
+            #  not hit DB
+            self.assertFalse(checker.has_perm("change_group", group2))
+            self.assertEqual(len(connection.queries), query_count)
         finally:
             settings.DEBUG = False
 
@@ -297,9 +329,10 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             user = User.objects.create(username='active_user', is_active=True)
             projects = \
                 [Project.objects.create(name='Project%s' % i)
-                    for i in range(2)]
-            for project in projects:
-                assign_perm("change_project", user, project)
+                    for i in range(3)]
+            assign_perm("change_project", user, projects[0])
+            assign_perm("change_project", user, projects[1])
+
             checker = ObjectPermissionChecker(user)
 
             # Prefetch permissions
@@ -307,7 +340,7 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             query_count = len(connection.queries)
 
             # Checking cache is filled
-            self.assertEqual(len(checker._obj_perms_cache), 2)
+            self.assertEqual(len(checker._obj_perms_cache), len(projects))
 
             # Checking shouldn't spawn any queries
             checker.has_perm("change_project", projects[0])
@@ -323,6 +356,11 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             checker.has_perm("change_project", projects[1])
             self.assertEqual(len(connection.queries), query_count)
 
+            # Checking for same model but other instance shouldn't spawn any queries
+            # Even though User doesn't have perms on projects[2], we still
+            #  should not hit DB
+            self.assertFalse(checker.has_perm("change_project", projects[2]))
+            self.assertEqual(len(connection.queries), query_count)
         finally:
             settings.DEBUG = False
 
@@ -337,8 +375,8 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             projects = \
                 [Project.objects.create(name='Project%s' % i)
                     for i in range(2)]
-            for project in projects:
-                assign_perm("change_project", user, project)
+            assign_perm("change_project", user, projects[0])
+
             checker = ObjectPermissionChecker(user)
 
             # Prefetch permissions
@@ -346,7 +384,7 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             query_count = len(connection.queries)
 
             # Checking cache is filled
-            self.assertEqual(len(checker._obj_perms_cache), 2)
+            self.assertEqual(len(checker._obj_perms_cache), len(projects))
 
             # Checking shouldn't spawn any queries
             checker.has_perm("change_project", projects[0])
@@ -361,7 +399,6 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             #  queries
             checker.has_perm("change_project", projects[1])
             self.assertEqual(len(connection.queries), query_count)
-
         finally:
             settings.DEBUG = False
 
@@ -371,20 +408,21 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             from django.db import connection
 
             ContentType.objects.clear_cache()
-            new_group = Group.objects.create(name='new-group')
+            group = Group.objects.create(name='new-group')
             projects = \
                 [Project.objects.create(name='Project%s' % i)
-                    for i in range(2)]
-            for project in projects:
-                assign_perm("change_project", new_group, project)
-            checker = ObjectPermissionChecker(new_group)
+                    for i in range(3)]
+            assign_perm("change_project", group, projects[0])
+            assign_perm("change_project", group, projects[1])
+
+            checker = ObjectPermissionChecker(group)
 
             # Prefetch permissions
             self.assertTrue(checker.prefetch_perms(projects))
             query_count = len(connection.queries)
 
             # Checking cache is filled
-            self.assertEqual(len(checker._obj_perms_cache), 2)
+            self.assertEqual(len(checker._obj_perms_cache), len(projects))
 
             # Checking shouldn't spawn any queries
             checker.has_perm("change_project", projects[0])
@@ -400,5 +438,10 @@ class ObjectPermissionCheckerTest(ObjectPermissionTestCase):
             checker.has_perm("change_project", projects[1])
             self.assertEqual(len(connection.queries), query_count)
 
+            # Checking for same model but other instance shouldn't spawn any queries
+            # Even though User doesn't have perms on projects[2], we still
+            #  should not hit DB
+            self.assertFalse(checker.has_perm("change_project", projects[2]))
+            self.assertEqual(len(connection.queries), query_count)
         finally:
             settings.DEBUG = False
